@@ -24,7 +24,11 @@
 
 #include "pangoatsui-private.h"
 #include <pango/pangocairo.h>
+#ifdef CAIRO_HAS_QUARTZ_FONT
+#include <cairo-quartz.h>
+#else
 #include <cairo-atsui.h>
+#endif
 
 G_BEGIN_DECLS
 
@@ -38,17 +42,15 @@ struct _PangoCairoATSUIFontMap
 {
   PangoATSUIFontMap parent_instance;
 
-  PangoRenderer *renderer;
-
   gdouble dpi;
 };
 
-GType pango_cairo_atsui_font_map_get_type (void);
+GType pango_cairo_atsui_font_map_get_type (void) G_GNUC_CONST;
 
 PangoATSUIFont *
 _pango_cairo_atsui_font_new (PangoCairoATSUIFontMap     *cafontmap,
 			     PangoContext               *context,
-			     const char                 *postscript_name,
+			     PangoATSUIFace             *face,
 			     const PangoFontDescription *desc);
 
 G_END_DECLS
